/****************************************************************
ConClient.C
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Description: This file is the new Manager Subordinate Configuration
Protocol implementation for the Client Side.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ConClient_C[] = "$Id: ConClient.C,v 1.10 1998/03/06 23:00:06 marc9442 Exp $";

#include "ConfigClient.h"
#include "Hummer_PG.h"

/**********
Function: ConfigClient::ConfigClient
Purpose:  Basic constructor. Initializes the timestamp.
**********/

ConfigClient::ConfigClient()
{
}


/**********
Function: ConfigClient::~ConfigClient
Purpose:  Basic destructor
**********/

ConfigClient::~ConfigClient()
{
}


/**********
Function: OpenServer
Purpose:  This function opens a socket connection to attach to the ConfigServer
          named in SubName, the return value will be the open OutSocket pointer
	  or NULL for an error.
***********/

ConfigClient::OpenServer(char *SubName)
{
  // Create new socket 
  ConnectedSocket = new sock_cli;
  ConnectedSocket->create();
  
  // Connect to remote machine
  if(ConnectedSocket->connect(SubName,CONFIG_PORT)){

    // Send authorinzation information to server, negotiate kerberos
    if (ConnectedSocket->send_auth()) {

      // Init the buffer
      char *InMessage;
      InMessage = new char[BUFFER_SIZE];
      bzero(InMessage,BUFFER_SIZE);

      // Read "Server is Ready string" to assure we are 
      // connectted to the right place
      ConnectedSocket->read(InMessage, BUFFER_SIZE); 
      
      if (strcmp(InMessage,"Configuration Server Ready")) {
	// If we didn't get the right string kill connection
	Print_Error("ConClient", "Configureation Server %s Not Ready",
		    SubName);
	ConnectedSocket->close();
	delete ConnectedSocket;
	ConnectedSocket = NULL;
      }
      delete [] InMessage;
    }
  }
  // If we can't connect die
  else {
    Print_Error("ConClient", "Can't Connect to %s", SubName);    
    
    ConnectedSocket->close();
    delete ConnectedSocket;
    ConnectedSocket = NULL;
  }
  
  // Return non-zero value if we connected successfully
  return (ConnectedSocket != NULL);
}


/**********
Function: ConfigClient::CloseServer()
Purpose:  This function closes the ConnectedSocket to the subordinate.
**********/
int ConfigClient::CloseServer()
{
  // End end of protocol mesage, and close out connection
  ConnectedSocket->write("GO RESTART SUBORDINATE;");
  ConnectedSocket->write("END;");
  ConnectedSocket->close();
  delete ConnectedSocket;
  ConnectedSocket = NULL;

  return 1;
}

/**********
Function: ConfigClient::PushLevelss
Purpose:  This function sends the non-inherited levels to the ConfigServer
          named in SubName.
**********/

int ConfigClient::PushLevels()
{
  // Memory for the query we are going to build
  char *query;
  query = new char[8192];

  // Open a connection to the DB, die if unsuccessful
  PgDatabase *leveldb = OpenDB();
  if (!leveldb) {
    delete leveldb;
    delete [] query;
    
    return 0;
  }

  // Select the non-inherited values from the levels table, die if no rows
  if (!QueryDB(&leveldb, "SELECT * FROM levels WHERE inh=0;")) {
    delete leveldb;
    delete [] query;
    
    return 0;
  }
  
  else {
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    sprintf(query, "DELETE FROM levels WHERE inh=1;");
    ConnectedSocket->write(query);

    // For every row returned create an SQL statement that will insert
    // the row on the server's side DB
    for (int i=0; i < leveldb->Tuples(); i++) {   
      sprintf(query, "INSERT INTO levels VALUES ( '%s', '%s', '%s', '%s', '%s', 1);",
	      leveldb->GetValue(i,0),  // start_addr
	      leveldb->GetValue(i,1),  // end_addr
	      leveldb->GetValue(i,2),  // trust
	      leveldb->GetValue(i,3),  // coop
	      leveldb->GetValue(i,4)); // integ
      
#ifdef DEBUG_QUERY
      Print_Error("ConClient", query);
#endif
      
      // Write out the query to the server
      ConnectedSocket->write(query);
    }

    // Close connection to DB, and free memory
    delete leveldb;
    delete [] query;
  }

  return 1;
}


/**********
Function: ConfigClient::PushFilters
Purpose:  This function sends the non-inherited filters to the ConfigServer
          named in SubName.
**********/

int ConfigClient::PushFilters()
{
  // Init some memory for the SQL query
  char *query;
  query = new char[8192];

  // Connect to the Database backend die if failure
  PgDatabase *filterdb = OpenDB();
  if (!filterdb) {
    delete filterdb;
    delete [] query;
    
    return 0;
  }
  
  // Get all the non-inherited values from the filter table, die if no tuples
  if (!QueryDB(&filterdb, "SELECT * FROM filter WHERE inh=0;")) {
    delete filterdb;
    delete [] query;
    
    return 0;
  }
  
  else {
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    sprintf(query, "DELETE FROM filter WHERE inh=1;");
    ConnectedSocket->write(query);

    // For every row returned from the select build a SQL statement 
    // to send to the server to insert into its DB
    for (int i=0; i < filterdb->Tuples(); i++) {   
    
      // Start of the SQL statement
      sprintf(query, "INSERT INTO filter VALUES ( ");

      // SQL doesn't like ( , , ,) to fill in Null values 
      // we must explicitly say were the NULLs go like ( NULL, NULL, NULL)

      // Check to see if it is an empty value, if so put a NULL in its place

      // expression
      if (!strcmp(filterdb->GetValue(i,0),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,0)); strcat(query, "', ");
      }
      
      // lots of these things

      // date_start
      if (!strcmp(filterdb->GetValue(i,1),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,1)); strcat(query, "', ");
      }
      
      // look at them all

      // end_date
      if (!strcmp(filterdb->GetValue(i,2),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,2)); strcat(query, "', ");
      }

      // there should be a better way around this problem

      // time_start
      if (!strcmp(filterdb->GetValue(i,3),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,3)); strcat(query, "', ");
      }

      // Reseason 86b that I don't like PostgreSQL

      // time_end
      if (!strcmp(filterdb->GetValue(i,4),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,4)); strcat(query, "', ");
      }

      // OK I'll make up some better comments.
      
      // trust
      if (!strcmp(filterdb->GetValue(i,5),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,5)); strcat(query, "', ");
      }

      // coop
      if (!strcmp(filterdb->GetValue(i,6),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,6)); strcat(query, "', ");
      }

      // integ
      if (!strcmp(filterdb->GetValue(i,7),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,7)); strcat(query, "', ");
      }

      // log
      if (!strcmp(filterdb->GetValue(i,8),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,8)); strcat(query, "', ");
      }
      
      // alert
      if (!strcmp(filterdb->GetValue(i,9),"")) strcat(query, "NULL, "); 
      else {
	strcat(query, "'"); strcat(query, filterdb->GetValue(i,9)); strcat(query, "', ");
      }
      
      // end the query with 1 for the inherited value
      strcat(query," 1);");
	      
#ifdef DEBUG_QUERY
      Print_Error("ConClient", query);
#endif
      
      // write the query to the server
      ConnectedSocket->write(query);
    }

    // Close out the connection to the DB, free memory
    delete filterdb;
    delete [] query;
  }

  return 1;
}

/**********
Function: ConfigClient::PushAccept
Purpose:  This function sends the non-inherited accept_list to the ConfigServer
          named in SubName.
**********/

int ConfigClient::PushAccept()
{
  // Create some space for the query
  char *query;
  query = new char[8192];

  // Connect to the DB back end
  PgDatabase *acceptdb = OpenDB();
  if (!acceptdb) {
    delete acceptdb;
    delete [] query;
    
    return 0;
  }
  
  // Select the non inherited values from the accept_list table
  // die if no tuples
  if (!QueryDB(&acceptdb, "SELECT * FROM accept_list WHERE inh=0;")) {
    delete acceptdb;
    delete [] query;
    
    return 0;
  }
  
  else { 
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    sprintf(query, "DELETE FROM accept_list WHERE inh=1;");
    ConnectedSocket->write(query);

    // For every row returned create a SQL statement to send to the
    // ConfigServer to insert into it's DB
    for (int i=0; i < acceptdb->Tuples(); i++) {   
      sprintf(query, "INSERT INTO accept_list VALUES ( '%s', '%s', '%s', 1);",
	      acceptdb->GetValue(i,0),  // filter
	      acceptdb->GetValue(i,1),  // start_addr
	      acceptdb->GetValue(i,2)); // end_addr
      
      // We're printing out the query here for help in debugging problems
#ifdef DEBUG_QUERY
      Print_Error("ConClient", query);
#endif

      // Send the query to the server
      ConnectedSocket->write(query);
    }

    // Close the DB connection, free memory
    delete acceptdb;
    delete [] query;
  }

  return 1;
}

/**********
Function: ConfigClient::PushKill
Purpose:  This function sends the non-inherited kill file list to the
          ConfigServer named in SubName.
**********/

int ConfigClient::PushKill()
{
  // Create some space for the query
  char *query;
  query = new char[8192];

  // Open connection to the DB backend
  PgDatabase *killdb = OpenDB();
  if (!killdb) {
    delete killdb;
    delete [] query;
    
    return 0;
  }
  
  // Select the vales from the kil_file that are non-inherited
  if (!QueryDB(&killdb, "SELECT * FROM kill_file WHERE inh=0;")) {
    delete killdb;
    delete [] query;
    
    return 0;
  }
  
  else { 
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    sprintf(query, "DELETE FROM kill_file WHERE inh=1;");
    ConnectedSocket->write(query);

    // For every row returned create an SQL statement that would 
    // insert the kill_file table on the server side DB
    for (int i=0; i < killdb->Tuples(); i++) {   
      sprintf(query, "INSERT INTO kill_file VALUES ( '%s', '%s', 1);",
	      killdb->GetValue(i,0),  // start_addr
	      killdb->GetValue(i,1)); // end_addr
      
#ifdef DEBUG_QUERY
      Print_Error("ConClient", query);
#endif

      // Write out query to server
      ConnectedSocket->write(query);
    }

    // close db connection, free memory
    delete killdb;
    delete [] query;
  }

  return 1;
}


/**********
Function: ConfigClient::PushForward
Purpose:  This function sends the non-inherited forward_list to the
          ConfigServer named in SubName.
**********/

int ConfigClient::PushForward()
{
  // Create memory for the query to be printed to
  char *query;
  query = new char[8192];
  
  // Connect to DB
  PgDatabase *forwarddb = OpenDB();
  if (!forwarddb) {
    delete forwarddb;
    delete [] query;
    
    return 0;
  }
  
  // Select the non-inherited values from the forward_list table
  // die if there are none
  if (!QueryDB(&forwarddb, "SELECT * FROM forward_list WHERE inh=0;")) {
    delete forwarddb;
    delete [] query;
    
    return 0;
  }
  
  else {  
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    sprintf(query, "DELETE FROM forward_list WHERE inh=1;");
    ConnectedSocket->write(query);

    // For every row returned create a SQL statement to insert it on the 
    // servers DB
    for (int i=0; i < forwarddb->Tuples(); i++) {   
      sprintf(query, "INSERT INTO forward_list VALUES ( '%s', '%s', 1);",
	      forwarddb->GetValue(i,0),  // filter
	      forwarddb->GetValue(i,1)); // host
      
#ifdef DEBUG_QUERY
      Print_Error("ConClient", query);
#endif

      // write out the query to the server
      ConnectedSocket->write(query);
    }

    // close connection to the DB
    delete forwarddb;
    delete [] query;
  }

  return 1;
}

/**********
Function: ConfigClient::PushConfig
Purpose:  This function sends the non-inherited Configuration to the 
          ConfigServer for each Subordinate in the DB.
**********/

int ConfigClient::PushConfig(void)
{
  // Open connection to the PostgreSQL database management server
  // die if the connection fails.
  PgDatabase *subdb = OpenDB();
  if (!subdb) {
    delete subdb;
    
    return 0;
  }
  
  // Get all teh subordinates to send our tables to.
  // die if there are none.
  if (!QueryDB(&subdb, "SELECT * FROM subordinates;")) {
    delete subdb;
    
    return 0;
  }
  
  else { 
    // For every subordinate send our tables
    for (int i=0; i < subdb->Tuples(); i++) {   
      // Open connection to the ConfigServer running on the subord.
      OpenServer( (char *) subdb->GetValue(i,1));

      // Push all our non-inherited values down 
      PushFilters();  // filters
      PushLevels();   // trust, coop, integ
      PushAccept();   // accept lists for filters
      PushForward();  // forward lists for filters
      PushKill();     // Kill file list

      CloseServer();  // End connection to subord
    }

    // close connection to DB
    delete subdb;
  }
  
  return 1;
}

