/****************************************************************
ConfigLoad.C
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Pete Neisen and Jamie Marconi
Description: The message handler is the bulk of the messaging server.
             It matches a message to a number of filters and then
	     uses these filters to process the message. 
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

static char ConfigLoad_C[] = "$Id: ConfigLoad.C,v 1.7 1998/03/06 23:00:07 marc9442 Exp $";

#include "MessageHandler.h"

/**********
Function: MessageHandler::LoadConfigFiles
Purpose:  This function calls LoadMessageFilter, LoadKills, and LoadLevel 
          to load the config files into the handler. Returns 1 all the time
	  now.
**********/

int MessageHandler::LoadConfigFiles(void)
{

  LoadMessageFilters();
  LoadLevel();
  LoadKills();
  
  return 1;
}



/**********
Function: MessageHandler::GetManager
Purpose:  This function returns the Manager of the hummer.  If there is
          no manager NULL is returned.
**********/

char *MessageHandler::GetManager()
{
  char * ManagerHostIP;
  ManagerHostIP = new char[IP_ADDRESS_SIZE];

  // Open New connection to DB, not a problem if one is already open
  PgDatabase *mandb = OpenDB();
  if (!mandb) {
    delete [] ManagerHostIP;
    
    return NULL;
  }
  
  // Perform the select on the table manager
  if (!QueryDB(&mandb,"SELECT * FROM manager;")) {
    delete mandb;
    delete [] ManagerHostIP;

    return NULL;
  }
   
  else {

    // If we were successful get the first entry only, even if more are listed
    strcpy(ManagerHostIP,  mandb->GetValue(0,0));
    
    delete mandb;

    return ManagerHostIP;
  }
}


/**********
Function: MessageHandler::GetSubord
Purpose:  This function returns the Subordinates of the hummer.  The number
          of subordinates is returned and the parameter Subord is the list
	  of hosts.
**********/

int MessageHandler::GetSubord(HostList **Subord)
{
  // Init memory
  char *SubordHostIP;
  SubordHostIP = new char[IP_ADDRESS_SIZE];
  HostList *Hosts;
  int number;
  
  *Subord = NULL;
  
  // Open new connection to DB, ok if others are already open
  PgDatabase *subdb = OpenDB();
  if (!subdb) {
    delete [] SubordHostIP;
    return 0;
  }
  
  // Perform select on the subordinate table
  if (!QueryDB(&subdb, "SELECT * FROM subordinates;")) {
    delete subdb;
    delete [] SubordHostIP;
    
    return 0;
  }

  else {
    // for each row returned from the query 
    for(int i=0; i < subdb->Tuples(); i++) {
      //move the data into temp storage
      strcpy(SubordHostIP, subdb->GetValue(i,1));

      // create a new node on a hostlist
      Hosts = new HostList;
	
      // Copy over the subordinate name
      strcpy(Hosts->StartHostIP,SubordHostIP); 
      strcpy(Hosts->EndHostIP,"0");
      
      // Move the pointers around
      Hosts->Next = *Subord;
      *Subord = Hosts;
    }
    
#ifdef DEBUG_SUBS
    // Print out the linked list to see if we got it right
    HostList *TempHost = *Subord;
    for (int i=0; i < subdb->Tuples(); i++) {
      Print_Error("Server", "Subord: %s", TempHost->StartHostIP);
      TempHost = TempHost->Next;
    }
#endif
        
    // return the number of rows we got
    number = subdb->Tuples();
    
    delete subdb;
  }

  delete[] SubordHostIP;
  
  return number;
}


/**********
Function: MessageHandler::LoadMessageFilter
Purpose:  This function fills the message filter container with message
          filter object that are taken from the configuration DB.
**********/

int MessageHandler::LoadMessageFilters(void)
{

  // Warning: This code is ugly
#ifdef POSTGRES
  
  // Init memory
  char *query;
  query = new char[8192];
  
  char *Token;
  char *ManagerHostIP;
  char *SubordHostIP;

  MessageFilter *Temp;
  HostList *TempList;
  HostList *PrevList;
  
  Token = new char[NORMAL_STRING_SIZE];

  //Make connection to the DB for the filter list
  PgDatabase *filterdb = OpenDB();
  if (!filterdb) {
    delete Token;
    delete query;
    
    return 0;
  }

  // Open a connection to the DB again for accept list queries
  PgDatabase *acceptdb = OpenDB();

  // Open a connection to the DB yet again for peer list queries
  PgDatabase *peerdb = OpenDB();
      
  // Open a connection to the DB yet yet again for froward list queries
  PgDatabase *forwarddb = OpenDB();
     
  // Select all the filters, even the inherited 
  if (!QueryDB(&filterdb, "SELECT * FROM filter;")) {

    delete filterdb;
    delete Token;
    delete query;

    return 0;
  }

  else { 
    // For every filter in the table returned
    for (int i=0; i < filterdb->Tuples(); i++) {   
      // Fill up the message filter object
      // HOLD ON TIGHT!

      // Create a new node MessageFilter
      Temp = new MessageFilter;
      
      // Copy the regular expression
      strcpy(Temp->RegExp, filterdb->GetValue(i,0));
      
      // Copy the Start date
      strcpy(Token, filterdb->GetValue(i,1));
      if(!strcmp(Token, "")) strcpy(Temp->StartDate,"~");
      else strcpy(Temp->StartDate, Token);
      
      // Copy the end date
      strcpy(Token, filterdb->GetValue(i,2));
      if(!strcmp(Token, "")) strcpy(Temp->EndDate,"~");
      else strcpy(Temp->EndDate, Token);
      
      // Copy the start time
      strcpy(Token, filterdb->GetValue(i,3));
      if(!strcmp(Token, "")) strcpy(Temp->StartTime,"~");
      else strcpy(Temp->StartTime, Token);
      
      // Copy the end time
      strcpy(Token, filterdb->GetValue(i,4));
      if(!strcmp(Token, "")) strcpy(Temp->EndTime,"~");
      else strcpy(Temp->EndTime, Token);
      
      // Copy the trust level
      strcpy(Token, filterdb->GetValue(i,5));
      if(!strcmp(Token, "")) strcpy(Temp->TrustLevel,"~");
      else strcpy(Temp->TrustLevel, Token);

      // copy the cooperation level
      strcpy(Token, filterdb->GetValue(i,6));
      if(!strcmp(Token, "")) strcpy(Temp->CoopLevel,"~");
      else strcpy(Temp->TrustLevel, Token);

      // Copy the integerity level
      strcpy(Token, filterdb->GetValue(i,7));
      if(!strcmp(Token, "")) strcpy(Temp->IntegLevel,"~");
      else strcpy(Temp->IntegLevel, Token);
      
      // copy the log boolean value
      strcpy(Token, filterdb->GetValue(i,8));  
      Temp->Log = Token[0];
      
      // Copy the alert boolean value
      strcpy(Token, filterdb->GetValue(i,9));
      Temp->Alert = Token[0];
  
      // Now comes the hard part, create a list of accept hosts,
      // forward hosts, and peer group modirators.

      // If the connection to the acceptdb was unsuccesful then we say
      // there were not accepts
      if (!acceptdb) {
	Temp->FromHostList = NULL;
      }
      
      else { // Connected to acceptdb

	// Create the query to find all accepts with the filter we are looking at 
	sprintf(query, "SELECT * FROM accept_list WHERE filter = '%s';", 
		Temp->RegExp);
	
	// run the query
	if (!QueryDB(&acceptdb, query)) {
	  Temp->FromHostList = NULL;
	}
  
	else { 
	  // Init some more memory
	  TempList = new HostList;
	  TempList->Next = NULL;
	  Temp->FromHostList = TempList;
	  PrevList = TempList;
	  
	  // For each accept row returned
	  for (int j=0; j < acceptdb->Tuples(); j++) {
	    // Copy the value in to temp space
	    strcpy(Token, acceptdb->GetValue(j,1));
      
	    // If its the alias manager find the manager and put it in
	    if(!strcmp(Token, "manager")) {

	      // Figure out who the manager is and add to list.
	      ManagerHostIP = GetManager();
	      // If no manager continue loop on next value
	      if ( !ManagerHostIP ) {
		continue;
	      }
	      
	      else {
		// Copy the manager name over to the list
		ConvertIP(ManagerHostIP,TempList->StartHostIP); 
		
		strcpy(TempList->EndHostIP, "0");
		
		// set up for the next node
		TempList->Next = new HostList;
		PrevList = TempList;
		TempList = TempList->Next;
		
		delete[] ManagerHostIP;
	      }
	    }
	    
	    // if its the alias subordinate find the subordinates and put them in
	    else if(!strcmp(Token, "subordinate")) {

	      // Init even more memory
	      SubordHostIP = new char[IP_ADDRESS_SIZE];
	      
	      HostList *Top;
	      HostList *Hosts;
	      // call the function to get subords.
	      int number = GetSubord(&Hosts);
	      Top = Hosts;
	      
	      // for each host in the subord list
	      for(int k=0; k < number; k++) {
		// copy it over to temp storage
		strcpy(SubordHostIP, Hosts->StartHostIP);

#ifdef DEBUG_SUBS
		Print_Error("Server", "Subordinate: %s", SubordHostIP);
#endif

		// delete the node on the subordante list
		Hosts = Hosts->Next;
		delete Top;
		Top = Hosts;
		
		// Copy the data into a new node 
		ConvertIP(SubordHostIP,TempList->StartHostIP); 
		strcpy(TempList->EndHostIP,"0");
		TempList->Next = new HostList;
		PrevList = TempList;
		TempList = TempList->Next;
		
	      }
	      
	      delete[] SubordHostIP;
	    }
	    
	    // It wasn't an alias so do the normal way
	    else {
	      // Copy the start address over
	      ConvertIP(Token,TempList->StartHostIP);

	      const char *DB_Field;
	      DB_Field = acceptdb->GetValue(j,2);

	      // If there was something in the end address deal with it
	      // other wise fill it will "0"
	      if (!strcmp(DB_Field, ""))
		strcpy(Token, "0");
	      else
		ConvertIP(DB_Field,Token);
	      
	      strcpy(TempList->EndHostIP, Token);
	      
	      // Set up for the next host node in the list
	      TempList->Next = new HostList;
	      PrevList = TempList;
	      TempList = TempList->Next;
	    }
	    
	  }
	
	  // Fix up the end of the list
	  PrevList->Next = NULL;
	  delete TempList;
    
	}
      }

      // if we didn't make a successful connection to the peer forward db
      // then we say that there are no forwards
      if (!peerdb) {
	Temp->ForwardPeerList = NULL;
      }
      
      else { // Connected to peerdb
	// Form the query to get the moderator address of the peer groups
	// selected  (its kinda long but makes sence in SQL)
	sprintf(query, "select pgroups.mod_addr, pgroups.pgroup from forward_peer, pgroups where forward_peer.filter = '%s' and forward_peer.pgroup = pgroups.pgroup;", 
		Temp->RegExp);
	
	// run the query
	if (!QueryDB(&peerdb, query)) {
	  // if no row were returned then the list is null.
	  Temp->ForwardPeerList = NULL;
	}
  
	else { 
	  // init some memory
	  PeerList *PrevPeer;
	  PeerList *TempPeer = new PeerList;
	  TempPeer->Next = NULL;
	  Temp->ForwardPeerList = TempPeer;
	  PrevPeer = TempPeer;
	  
	  // For every row returned
	  for (int j=0; j < peerdb->Tuples(); j++) {
	    
	    // Copy over the data into a peer list
	    strcpy(TempPeer->ModHostIP, peerdb->GetValue(j,0));
	    strcpy(TempPeer->GroupName, peerdb->GetValue(j,1));
	    
	    // set up for the next node
	    TempPeer->Next = new PeerList;
	    PrevPeer = TempPeer;
	    TempPeer = TempPeer->Next;
	  }
	  
	  // clean up the end of the list
	  PrevPeer->Next = NULL;
	  delete TempPeer;
	}
      }
      

      // If we didn't make a successful connection to the forward db
      // we assume that there were no forwards
      if (!forwarddb) {
	Temp->ForwardHostList = NULL;
      }
      
      else {
	// Create the sql command 
	sprintf(query, "SELECT * FROM forward_list WHERE filter = '%s';", 
		Temp->RegExp);
  
	// run the query
	if(!QueryDB(&forwarddb, query)) {
	  // if no rows returned then the list is empty
	  Temp->ForwardHostList = NULL;
	}
	
	else {
	  // Init some memory
	  TempList = new HostList;
	  TempList->Next = NULL;
	  Temp->ForwardHostList = TempList;
	  PrevList = TempList;
	  
	  // for every row returned in the query
	  for (int j=0; j < forwarddb->Tuples(); j++) {      
	    
	    // Copy the hostname to temp space
	    strcpy(Token, forwarddb->GetValue(j,1));
	    
	    // if it was the alias manager
	    if(!strcmp(Token, "manager")){
	      // Figure out who the manager is and add to list.
	      
	      ManagerHostIP = GetManager();
	      
	      // If we got the manager copy of the data to a new node
	      if (ManagerHostIP) {
		strcpy(TempList->StartHostIP,ManagerHostIP); 
		strcpy(TempList->EndHostIP,"~");
		TempList->Next = new HostList;
		PrevList = TempList;
		TempList = TempList->Next;
		
		delete[] ManagerHostIP;
	      }   
	      
	      else {
		continue;
	      }
	    }
	    
	    // if it was the alias subordinate
	    else if(!strcmp(Token, "subordinate")){
	      // Figure out the subordiantes and add them to the list.
	      
	      SubordHostIP = new char[IP_ADDRESS_SIZE];
	      
	      HostList *Top;
	      HostList *Hosts;

	      // get the subords
	      int number = GetSubord(&Hosts);
	      Top = Hosts;
	      
	      // for each subord 
	      for(int k=0; k < number; k++) {

		// Copy the name to temp space
		strcpy(SubordHostIP, Hosts->StartHostIP);
		
#ifdef DEBUG_SUBS
		Print_Error("Server", "Subordinate: %s", SubordHostIP);
#endif
		// delete it from the top of the subord list
		Hosts = Hosts->Next;
		delete Top;
		Top = Hosts;
		
		// Copy over the subord to the host list
		strcpy(TempList->StartHostIP,SubordHostIP); 
		strcpy(TempList->EndHostIP,"~");
		TempList->Next = new HostList;
		PrevList = TempList;
		TempList = TempList->Next;
		
	      }
	      
	      delete[] SubordHostIP;
	    }
	    
	    // it wasn't an alias so do it normally.
	    else{
	      // Copy over the address, and set up the next node
	      strcpy(TempList->StartHostIP, Token);
	      strcpy(TempList->EndHostIP,"~");
	      TempList->Next = new HostList;
	      PrevList = TempList;
	      TempList = TempList->Next;
	    }
	  }
	
	  // fix up the end of the list
	  PrevList->Next = NULL;
	  delete TempList;      
	}
      }
      
      Filters.AddFilter(Temp);  // Add the filter to the container. finally!!
    }
  }
  
  delete [] query;
    
  // Free memory, and close db connections.
  delete[] Token;
  delete forwarddb;
  delete peerdb;
  delete acceptdb;
  delete filterdb;
#endif
  return 1;
}


/**********
Function: MessageHandler::LoadLevel
Purpose:  This function loads the level container similar to how the function
          above loads the message filter container. 
**********/

int MessageHandler::LoadLevel(void)
{
#ifdef POSTGRES	
  char *Token;
  char *ManagerHostIP;
  char *SubordHostIP;
  int TrustLevel;
  int CoopLevel;
  int IntegLevel;
  Level *Temp;

  // Set default level to 5 
  Levels.SetDefaultTrust(DEFAULT_TRUST);
  Levels.SetDefaultCoop(DEFAULT_COOP);
  Levels.SetDefaultInteg(DEFAULT_INTEG);

  // Allocate memory
    
  Token = new char[NORMAL_STRING_SIZE];

  // Open a connection to the db
  PgDatabase *leveldb = OpenDB();
  if (!leveldb) {
    return 0;
  }

  // get all the levels from the table
  if (!QueryDB(&leveldb, "SELECT * FROM levels;")) {
    delete leveldb;

    return 0;
  }

  else {
    const char *DB_Field;
    // for every row returned from the query
    for (int i=0; i < leveldb->Tuples(); i++) {
      // Copy the host name over to temp space
      DB_Field = leveldb->GetValue(i,0);
      strcpy(Token, DB_Field);
      
      // if its the alias manager
      if(!strcmp(Token, "manager")) {
	// Figure out who the manager is and add to list.
	
	ManagerHostIP = GetManager();
	
	if (ManagerHostIP) {
	  // Copy the data over to a new node
	  Temp = new Level;
	  
	  ConvertIP(ManagerHostIP,Temp->StartHostIP);
	 
	  strcpy(Temp->EndHostIP,"0");

	  // Copy the Trust
	  DB_Field = leveldb->GetValue(i,2);
	  Temp->TrustLevel = DB_Field[0];

	  // Copy the cooperation
	  DB_Field = leveldb->GetValue(i,3);
	  Temp->CoopLevel = DB_Field[0];

	  // Copy the integerity
	  DB_Field = leveldb->GetValue(i,4);
	  Temp->IntegLevel = DB_Field[0];
	  
	  // Add to level list.
	  Levels.AddLevel(Temp);
	  
	  delete[] ManagerHostIP;
	}
      }
      
      // if its the alias subordinate
      else if ( !strcmp(Token, "subordinate") ) {
	// Figure out the subordiantes and add to list.
	
	SubordHostIP = new char[IP_ADDRESS_SIZE];
	
	// get the trust 
	DB_Field = leveldb->GetValue(i,2);
	TrustLevel = DB_Field[0];
	
	// get the cooperation
	DB_Field = leveldb->GetValue(i,3);
	CoopLevel = DB_Field[0];
	
	// get the integrity
	DB_Field = leveldb->GetValue(i,4);
	IntegLevel = DB_Field[0];
	
	HostList *Top;
	HostList *Hosts;
	// Get the subordinates
	int number = GetSubord(&Hosts);
	Top = Hosts;
	
	// For each subordnate
	for(int j=0; j < number; j++) {
	  // Copy the subord name over to temp space
	  strcpy(SubordHostIP, Hosts->StartHostIP);
	  
#ifdef DEBUG_SUBS
	  Print_Error("Server", "Subordinate: %s", SubordHostIP);
#endif

	  // delete the top of the subordnate list
	  Hosts = Hosts->Next;
	  delete Top;
	  Top = Hosts;
	  
	  Temp = new Level;

	  // Create a new node for the subordante and add the levels
	  ConvertIP(SubordHostIP,Temp->StartHostIP);
	  strcpy(Temp->EndHostIP,"0");
	  Temp->TrustLevel = TrustLevel;
	  Temp->CoopLevel = CoopLevel;
	  Temp->IntegLevel = IntegLevel;
	  
	  // Add to level list.
	  Levels.AddLevel(Temp);
	  
	}
	
	delete[] SubordHostIP;
      }
      
      // it wasn't an alias
      else {
	// Handle the trust normally
	
	Temp = new Level;
	// Copy over the start address
	ConvertIP(Token,Temp->StartHostIP);
	
	DB_Field = leveldb->GetValue(i,1);
	
	// if the end address is not nothing then copy it over
	if (!strcmp(DB_Field, ""))
	  strcpy(Token, "0");
	else 
	  ConvertIP(DB_Field,Token);

       	strcpy(Temp->EndHostIP, Token);

	// copy the trust
	DB_Field = leveldb->GetValue(i,2);
	Temp->TrustLevel = DB_Field[0];

	// copy the cooperation
	DB_Field = leveldb->GetValue(i,3);
	Temp->CoopLevel = DB_Field[0];

	// copy the integrity
	DB_Field = leveldb->GetValue(i,4);
	Temp->IntegLevel = DB_Field[0];

	// Add to level list.
	Levels.AddLevel(Temp);
      }
    }
    
    delete leveldb;
  }

  // Free memory
  delete[] Token;
#endif /* POSTGRES */
  
  return 1;
}

/**********
Function: MessageHandler::LoadKills
Purpose:  This function loads the kill file container similar to how the 
          function above loads the message filter container. 
**********/

int MessageHandler::LoadKills(void)
{
#ifdef POSTGRES	
  char *Token;
  char *ManagerHostIP;
  char *SubordHostIP;
  HostList *Temp;

  // Delete old Kill File
  HostList *DelPtr;
    
  Temp = Kills;
  while (Temp != NULL) {
    DelPtr = Temp;
    Temp = Temp->Next;
    delete DelPtr;
  }
  Kills = NULL;

  // Allocate memory
  
  Token = new char[NORMAL_STRING_SIZE];

  // Connect to the database
  PgDatabase *killsdb = OpenDB();
  if (!killsdb) {
    return 0;
  }
    
  // all the entries in the kill file.
  if (!QueryDB(&killsdb, "SELECT * FROM kill_file;")) {
    delete killsdb;

    return 0;
  }

  else {
    const char *DB_Field;
    // for every row returned 
    for (int i=0; i < killsdb->Tuples(); i++) {
      // Copy the host over to temp space
      DB_Field = killsdb->GetValue(i,0);
      strcpy(Token, DB_Field);
      
      // if its the alias manager
      if(!strcmp(Token, "manager")) {
	// Figure out who the manager is and add to list.
	
	ManagerHostIP = GetManager();
	
	if (ManagerHostIP) {
	  Temp = new HostList;
	  
	  // copy the data over
	  ConvertIP(ManagerHostIP,Temp->StartHostIP); 
	  strcpy(Temp->EndHostIP,"0");

	  // Add to kill list.
	  Temp->Next = Kills;
	  Kills = Temp;
	  
	  delete[] ManagerHostIP;
	}
      }
      
      // if its the alias subordinate
      else if ( !strcmp(Token, "subordinate") ) {
	// Figure out the subordiantes and add to list.
	HostList *Hosts;
	int number = GetSubord(&Hosts);
	
	// for each subordinate
	for (int k=0; k < number; k++) {
	  Temp = new HostList;
	  
	  // copy the host name over to a new node
	  ConvertIP(Hosts->StartHostIP, Temp->StartHostIP);
	  strcpy(Temp->EndHostIP,"0");
	  
	  Temp->Next = Kills;
	  Kills = Temp;

	  // delete the top of the subordinate list
	  DelPtr = Hosts;
	  Hosts = Hosts->Next;
	  delete DelPtr;
	}

      }

      // if its not an aliase
      else {
	// Handle the host normally
	
	Temp = new HostList;
	
	// copy over the host name
	ConvertIP(Token,Temp->StartHostIP);
	
	DB_Field = killsdb->GetValue(i,1);
	
	// if the end address is not empty copy it over
	if (!strcmp(DB_Field, ""))
	  strcpy(Token, "0");
	else
	  ConvertIP(DB_Field,Token);

	strcpy(Temp->EndHostIP, Token);

	// Add to kill list.
	Temp->Next = Kills;
	Kills = Temp;
      }
    }
    
    delete killsdb;
  }

  // Free memory
  delete[] Token;
#endif /* POSTGRES */

#ifdef DEBUG_KILLS
  // Print out the kills list to be sure its working
  Print_Error("LoadKills", "Starting Printing");
  Temp = Kills;
  while (Temp != NULL) {
    Print_Error("LoadKills", "kill_file: %s %s", Temp->StartHostIP, Temp->EndHostIP);
    Temp = Temp->Next;
  }
#endif  

  return 1;
}

