#! &PERL& -I&INCDIR&
# -*- perl -*-


 ##############################################################
 #	Hummingbird 
 # 	Dean Polla
 #	06/21/97
 #
 #	This alert is designed to send out a warning when the 
 #	remaining disk space gets too small on a user defined
 #	File System.
 #
 #	The "bdf" for the hp's command is used to calculate the 
 #      information.  Or "du" on Solaris.
 #	The alert is based on the %used and the user defined
 #	%maxPercentage to be used.  Or the alert is based on 
 #	the space available and the user defined minimum space
 #	available.
 #
 #      Requires command line parameters
 ##############################################################

#
# Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

forktoBg($0);

use Toollib;
use Datelib qw(hhmmss yyyymmdd $datetime);
use Getopt::Long;


%optctl = (perc		=> \$maxPercentage, 
	   time		=> \$sleepTime,
	   alert	=> \$alerts,
	   diff		=> \$maxPercDiff,
	   space	=> \$minSpaceAvailable,
	   file		=> \$FILESYSTEM);

$result = GetOptions(\%optctl, "perc=i", "time=i", "alert=i",
		     "diff:i", "space:i", "file:s");


if ($result == 0)     { print "Error in command line arguments\n"; }
if ($FILESYSTEM        eq "") { $FILESYSTEM = $ENV{"HOME"};  }
if ($minSpaceAvailable eq "") { $minSpaceAvailable = 100000; }
if ($maxPercDiff eq "")       { $maxPercDiff = 1; }	

if (($maxPercentage    eq "") || ($sleepTime eq "") || 
    ($alerts           eq "")) { 
      print "DiskAudit -perc <Percentage> -time <Sleep> -alert <# Alerts>\n";
      print "Using defaults. \n";
      $maxPercentage = 90;
      $sleepTime = 30;
      $alerts = 5;
    }

$basealerts 	= $alerts;
$firstpass 	= 1; 
$bound 		= 1; 
$sendflag 	= 1;


while (!$do_exit) {

 &diskAudit();
 $bound 	= 0;
 $firstpass 	= 1;
 $alerts 	= $basealerts;
 $sendflag 	= 1;
}

&delpid();


#################################################################
# This subroutine will monitor the bdf.				# 
#################################################################

sub diskAudit{

 while (($alerts)&&(!$do_exit)) {

  # Execute the disk usage command with the specified filesystem.
    $buffer = `&BDF_TYPE& $FILESYSTEM`;

  # Parse the data into a buffer.
    ($g1,$g2,$g3,$g4,$g5,$g6,$g7,$fileSystem,$kbytes,$used,$avail,
    $percUsed,$mountedOn) = split(" ", $buffer);
    chop($percUsed);

   if ($firstpass) {
    $basePerc = $percUsed;
    $firstpass = 0;
   }
    
   &percentageCheck;

  # If the percentage used is greater than the user defined percentage 
  # send an alert.  Or if the space available is less than the user 
  # defined minimum send an alert.
    if ((($percUsed >= $maxPercentage) || ($avail < $minSpaceAvailable)) && 
	 ($bound)) {
      $messageflag = 1;
      &sendAlert($messageflag);
    }
    elsif ($percDiff >= $maxPercDiff) {
      $messageflag = 2;
      &sendAlert($messageflag);
      if ($sendflag) {
	system "loadAlert";
	$sendflag = 0;
      }
    }   
        
 sleep $sleepTime;
 } #while
}



##################################################################
# Sends ALERT to ToolI
##################################################################
  
sub sendAlert {
    local($flag) = @_;
    
    $date =  yyyymmdd();
    $time =  hhmmss();
    $alerts--;
    
    if ($flag == 1) {
	$message = join ('', "ALERT   Disk Space on ", $fileSystem, 
			 " has reached ", $percUsed, "% used with ",
			 $avail, " kbytes available.");  
    }
    else {
	$message = join ('', "ALERT   Disk Space on ", $fileSystem, 
			 " has risen ", $percDiff, "% since ", $datetime,
			 ".  Now it's ", $percUsed,"%");  
    }	

    ToolI($message, $date, $time, $hostname, $logfile, "DiskAudit");
}  



##################################################################
# Calculate percentage difference and check percentage to see 
# if it has dropped.
##################################################################

sub percentageCheck {
    $percDiff = ($percUsed - $basePerc); 
    
    if ($percDiff < 0) { 
	$basePerc = $percUsed;
    }
}
