#
# Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

package Toollib;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(ToolI atoiMonth forktoBg delpid handler $sleepTime $hostname
	     $host $logfile $do_exit);


#############################################################
# Global Variables to all Tools
#############################################################
$logfile = "null";

use Pg;
use Sys::Hostname;
$host = hostname();

$hostname = (gethostbyname $host)[0];
$sleepTime = 5;

#############################################################
# Calls ToolI with parameters
#############################################################
sub ToolI {
    local($message, $date, $time, $hostname, $file, $tool) = @_;

    my $comline = "&TOOLI&/ToolI";
    if ($message ne "") {
	$comline = $comline . " -m \"$message\"";
    }
    if ($date ne "") {
	$comline = $comline . " -d \"$date\"";
    }
    if ($time ne "") {
	$comline = $comline . " -t \"$time\"";
    }
    if ($hostname ne "") {
	$comline = $comline . " -h \"$hostname\"";
    }
    if ($file ne "") {
	$comline = $comline . " -f \"$file\"";
    }
    if ($tool ne "") {
	$comline = $comline . " -s \"$tool\"";
    }

    #print "$comline\n";
    system "$comline";
}

#############################################################
# Convert the month from an alpha to numeric representation
#############################################################
sub atoiMonth {
    local($month) = @_;
    
    $month =~ s/Jan/01/;        $month =~ s/Feb/02/;
    $month =~ s/Mar/03/;        $month =~ s/Apr/04/;
    $month =~ s/May/05/;        $month =~ s/Jun/06/;
    $month =~ s/Jul/07/;        $month =~ s/Aug/08/;
    $month =~ s/Sep/09/;        $month =~ s/Oct/10/;   
    $month =~ s/Nov/11/;        $month =~ s/Dec/12/;

    $month;
}



#############################################################
# Fork to the background
#############################################################
sub forktoBg {
    local($sig) = @_;
    
    # Close term interaction 
    open(STDOUT, ">> &LOG&/Hummer.log");
    open(STDERR, ">> &LOG&/Hummer.log");
    open(STDIN,  "/dev/null");
    
    if ($pid = fork) {
	print "Forking:   @_ \n";
        exit(0);
    }
    else {
      my $mypid = $$;
      my $conn = Pg::connectdb("dbname = hummer_$host");
      $msg = $conn->errorMessage;
      ( PGRES_CONNECTION_OK eq $conn->status ) || die "Can't Connect to DB: $msg\n";
      my $result = $conn->exec("insert into pid values ('$0', $mypid);");
      $msg = $conn->errorMessage;
      ( PGRES_COMMAND_OK eq $result->resultStatus) || die "Insert Failed: $msg\n";
    }
}



#############################################################
# delete pid of tool from pid table
#############################################################
sub delpid {
  my $conn = Pg::connectdb("dbname = hummer_$host");
  $msg = $conn->errorMessage;
  ( PGRES_CONNECTION_OK eq $conn->status ) || die "Can't Connect to DB: $msg\n";
  my $result = $conn->exec("delete from pid where pid = $$;");
  $msg = $conn->errorMessage;
  ( PGRES_COMMAND_OK eq $result->resultStatus) || die "Delete Failed: $msg\n";
}


#############################################################
# Signal handler kills background process
#############################################################
sub handler { 
    local($sig) = @_;

    print "Caught a SIG$sig--shutting down:  $0\n";

    $do_exit = 1;
}

$SIG{INT} = \&handler;      # Set signal handler to die on request


$do_exit = 0;

1;

