/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// Modifications Complete 05/14/98 Dean Polla hummer@cs.uidaho.edu

// ***************************************************************************
// Atom Class
// 
//
// Class Description:  
// ------------------
// This class handles the system level execution and termination of the tools
// registered with Atom.
//
// Methods
// -------
//  public void startProcess(String dir, String command) 
//  public void startDaemon(String dir, String command) 
//  public void startSclTool(String sclType) 
//  public void stopTool(String sclType, String tool) 
//
// ***************************************************************************

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;


public class ToolExecution {

    
  // *********************************************************************************
  // public void startProcess(String dir, String command)
  //
  // Start a single Process on the system.  A process blocks and waits for output.
  //
  // *********************************************************************************
    
  public void startProcess(String dir, String command) {
	
      String cmdline = dir+command;
      
      // Garbage collection
      Runtime resource = Runtime.getRuntime();
      resource.gc();
      
      try {
	  String pipeio;        // stdout collected from exec
	  
	  // Create Runtime and Process object and exec the process
	  Process process = Runtime.getRuntime().exec(cmdline);
	    
	  // Notify System Status
	  Atom.systemStatusTextArea.append("Exec: "+ cmdline +"\n");

	  // Capture output from process
	  BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
	  
	  while ((pipeio = input.readLine()) != null) {
	      System.out.println(pipeio);
	}
	  
	  input.close();
      }
      
      catch (Exception err) {
	  System.out.println("Tool Execution Failed: ");
	  System.out.println(err.toString());
	  //	  err.printStackTrace();
	  return;
      }
  }  // startProcess


  // *********************************************************************************
  // public void startProcess(String dir, String command)
  //
  // Start a daemon process on the system.  A daemon process is non-blocking and
  // does not wait for output.
  //
  // *********************************************************************************
    
  public void startDaemon(String dir, String command) {
	
      String cmdline = dir+command;
      
      // Garbage collection
      Runtime resource = Runtime.getRuntime();
      resource.gc();
      
      try {
	  String pipeio;        // stdout collected from exec
	  
	  // Create Runtime and Process object and exec the process
	  Process process = Runtime.getRuntime().exec(cmdline);
	    
	  // Notify System Status
	  Atom.systemStatusTextArea.append("Exec: "+ cmdline +"\n");
      }
      
      catch (Exception err) {
	  System.out.println("Tool Execution Failed: ");
	  System.out.println(err.toString());
	  //	  err.printStackTrace();
	  return;
      }
  }  // startDaemon


  // *********************************************************************************
  // public void startSclTool(String sclType)
  //
  // Start processes/daemons on the system according to the security level file.  The
  // security level file contains information on whether the application is a process
  // or a a daemon.
  //
  // *********************************************************************************
 
  public void startSclTool(String sclType) {
      String line;

      // File location 
      String sclfile   = Atom.sclLevelDir+sclType;

      // VALIDITY check
      sclfile = sclfile.replace(' ', '_');
      File checkFile = new File(sclfile);
      if (!(checkFile.exists())) {
	  Atom.systemStatusTextArea.append("* No Security Level Assignments.\n");
	  Atom.systemStatusTextArea.append("  SCL --> Assignment.\n");
	  return;
      }
      
      // Garbage collection
      Runtime resource = Runtime.getRuntime();
      resource.gc();
      
      try {
	  String pipeio;        // stdout collected from exec
	  String cmdline;       // application to be executed
	  
	  File file = new File(sclfile);
	  BufferedReader   infile = new BufferedReader(new FileReader(file));
	  
	  while ((line = infile.readLine()) != null) {
	      
	      StringTokenizer inputString = new StringTokenizer(line,"^");
	      
	      cmdline = inputString.nextToken();
	      cmdline += inputString.nextToken();
	      String processType = inputString.nextToken();
	      System.out.println(cmdline);

	      // Create Runtime and Process object and exec the process
	      Process process = Runtime.getRuntime().exec(cmdline);
	
	      // Notify System Status
	      Atom.systemStatusTextArea.append("Exec: "+ cmdline +"\n");
	      
	      if (processType.equals(ToolInterface.process)) {
		  
		  //Capture output from process
		  BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
		  
		  while ((pipeio = input.readLine()) != null) {
		      System.out.println(pipeio);
		  }
		  input.close();
	      }
	  } // while
      } 
      catch (Exception err) {
	  System.out.println("Tool Execution Failed: ");
	  System.out.println(err.toString());
	  //	  err.printStackTrace();
	  return;
      }
  }  // startSclTool
    


  // *********************************************************************************
  // public void stopScl(String sclType)
  //
  // Method Description:  This method calls the stopTools (perl) script through the Runtime 
  // Object variable.  The stopTools script accepts command line arguments.
  //
  // Perl StopTool Script:  
  //
  //     Command Line arguments:
  //        -cmd  <process name> 
  //        -file <security level file name>
  // 
  // ********************************************************************************* 

  public void stopTool(String sclType, String tool) {
      String cmdline = "";
      String msg = "";

      if (tool == null) {
	  String file     = Atom.sclLevelDir+sclType;
	  file = file.replace(' ', '_');
	  cmdline = Atom.stopScript+" -file "+file; 
	  msg = "Kill Security Level: "+sclType+"\n";

	  //VALDITIY check
	  File checkFile = new File(file);
	  if (!(checkFile.exists())) {
	      Atom.systemStatusTextArea.append("* No Security Level Assignments.\n");
	      Atom.systemStatusTextArea.append("  SCL --> Assignment.\n");
	      return;
	  }
      }
      else {
	  cmdline = Atom.stopScript+" -cmd "+tool;
	  msg = "Kill: "+cmdline+"\n";
      }


      // Garbage collection
      Runtime resource = Runtime.getRuntime();
      resource.gc();
      
      try {
	  String pipeio;        // stdout collected from exec
	  
	  // Create Runtime and Process object and exec the process
	  Process process = Runtime.getRuntime().exec(cmdline);
	    
	  // Notify System Status
	  Atom.systemStatusTextArea.append(msg);

	  // Capture output from process
	  BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
	  
	  while ((pipeio = input.readLine()) != null) {
	      System.out.println(pipeio);
	  }
	  
	  input.close();
      }
      catch (Exception err) {
	  System.out.println("Tool Termination Failed: ");
	  System.out.println(err.toString());
	  System.err.println("ToolExecution.stopTool()");
	  return;
      } 
  } // stopTool
    
    
}
